@@ -1,21 +1,18 @@
-extern "C" {
 #include "config.h"
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <gtk/gtk.h>
-#include <libgimp/gimp.h>
-#include <libgimp/gimpui.h>
-#include <libgimp/gimpimage_pdb.h>
-#include "plugin-intl.h"
+#include <glib.h>
 
+extern "C" {
+#include "compter.h"
 #include "main.h"
-#include "render.h"
 #include "texturize.h"
 }
 #include "graph.h"
 
-#define MAX_CAPACITY 16383 //la moiti du plus grand short, (captype est short dans graph.h)
+#define MAX_CAPACITY 16383 // Half of the largest short, (captype is short in graph.h)
 #define REMPLI    1
 #define CUT_NORTH 2
 #define CUT_WEST  4
@@ -23,22 +20,19 @@ extern "C" {
 #define HAS_CUT_WEST(r)  (r) & CUT_WEST
 
 // ||pixel1 - pixel2||^2
-// Exprimentalement, le carr semble mieux marcher que la norme 2.
-inline Graph::captype
-cost (guchar * pixel1, guchar * pixel2, int channels)
-{
+// From experience, squares seem to work better than another type of norm.
+inline Graph::captype cost (guchar * pixel1, guchar * pixel2, int channels) {
   int diff, result = 0;
   for (int c = 0; c < channels; c++){
     diff = pixel1[c] - pixel2[c];
     result += diff*diff;
   }
   return (result/24);
-  // Il faut diviser au moins par 24 sinon on risque de renvoyer plus que MAX_CAPACITY.
+  // We need to divide at least by 24, or we might return more than
+  // MAX_CAPACITY.
 }
 
-inline Graph::captype
-gradient (guchar * pixel1, guchar * pixel2, int channels)
-{
+inline Graph::captype gradient (guchar * pixel1, guchar * pixel2, int channels) {
   int diff, result = 0;
   for (int c = 0; c < channels; c++){
     diff = pixel1[c] - pixel2[c];
@@ -47,68 +41,63 @@ gradient (guchar * pixel1, guchar * pixel2, int channels)
   return ((Graph::captype) sqrt(result));
 }
 
-//si les quatre arguments de edge_weight sont crits dans le code sur deux lignes
-//alors les mmes coordonnes de pixels sont sur une mme ligne,
-//les mmes images d'origine du pixel sur une mme colonne.
-inline Graph::captype
-edge_weight (int channels,
-             guchar * im1_pix1, guchar * im2_pix1,
-             guchar * im1_pix2, guchar * im2_pix2)
-{
+// When we write the four arguments to edge_weight on two lines of code,
+// we try to always align things (pixel VS image) so that it makes sense.
+inline Graph::captype edge_weight (int channels,
+                                   guchar * im1_pix1, guchar * im2_pix1,
+                                   guchar * im1_pix2, guchar * im2_pix2) {
   return ((cost(im1_pix1,im2_pix1,channels) + (cost(im1_pix2,im2_pix2,channels)))
           / (gradient(im1_pix1,im1_pix2,channels) + gradient(im2_pix1,im2_pix2,channels) +1));
 }
 
-inline void
-paste_patch_pixel_to_image(int width_i, int height_i, int width_p, int height_p,
-                           int x_i, int y_i, int x_p, int y_p,
-                           int channels,
-                           guchar * image, guchar * patch) {   //,
-                           //guchar * coupe_h_here, guchar * coupe_v_here){
+inline void paste_patch_pixel_to_image(int width_i, int height_i, int width_p, int height_p,
+                                       int x_i, int y_i, int x_p, int y_p,
+                                       int channels,
+                                       guchar * image, guchar * patch) {
   int k;
-  for (k = 0; k < channels; k++)
+  for (k = 0; k < channels; k++) {
     image[(y_i * width_i + x_i) * channels + k] = patch[(y_p * width_p + x_p) * channels + k];
-  /*
-  if (y_i < height_i - 1 && y_p < height_p - 1){
-    for(k = 0; k < channels; k++)
+    /*
+      Might become useful again if we start taking old cuts into account again.
+      if (y_i < height_i - 1 && y_p < height_p - 1){
+      for(k = 0; k < channels; k++)
       coupe_v_here[((y_i + 1) * width_i + x_i) * channels + k] = patch[((y_p + 1) * width_p + x_p) * channels + k];
-  }
-  if (x_i < width_i - 1 && x_p < width_p - 1) {
-    for(k = 0; k < channels; k++)
+      }
+      if (x_i < width_i - 1 && x_p < width_p - 1) {
+      for(k = 0; k < channels; k++)
       coupe_h_here[(y_i * width_i + x_i + 1) * channels + k] = patch[(y_p * width_p + x_p + 1) * channels + k];
+      }
+    */
   }
-  */
 }
 
-void
-decoupe_graphe (int* patch_posn,
-                int width_i, int height_i, int width_p, int height_p,
-                int channels,
-                guchar  **rempli,
-                guchar   *image, guchar * patch,
-                guchar   *coupe_h_here, guchar * coupe_h_west,
-                guchar   *coupe_v_here, guchar * coupe_v_north,
-                gboolean  make_tileable, gboolean invert)
-{
+void decoupe_graphe (int* patch_posn,
+                     int width_i, int height_i, int width_p, int height_p,
+                     int channels,
+                     guchar  **rempli,
+                     guchar   *image, guchar * patch,
+                     guchar   *coupe_h_here, guchar * coupe_h_west,
+                     guchar   *coupe_v_here, guchar * coupe_v_north,
+                     gboolean  make_tileable, gboolean invert) {
+
 ////////////////////////////////////////////////////////////////////////////////
-// Dclaration des variables
+// Variable declaration.
   gint k, x_p, y_p, x_i, y_i;// nb_sommets, sommet_courant; // Compteurs
   gint real_x_i, real_y_i;
   gint x_inf, y_inf, x_sup, y_sup;
-  int chute_patch_posn[2];
-  Graph * graphe = new Graph(); // Le graphe  couper
-  Graph::node_id node_of_pixel[width_p * height_p]; // Le noeud du graph auquel correspond un pointeur.
+  Graph * graphe = new Graph(); // Le graphe à couper
+  Graph::node_id *node_of_pixel = (void **) calloc (width_p * height_p, sizeof (Graph::node_id)); // Le noeud du graph auquel correspond un pointeur.
   for (k=0; k<width_p * height_p; k++) node_of_pixel[k] = NULL;
 
-  Graph::captype poids; // Pour calculer le poids d'un arc avant de le dclarer  Graph:add_edge
-  Graph::node_id first_node = NULL, node_sommet_courant; 
+  Graph::captype poids; // Pour calculer le poids d'un arc avant de le déclarer à Graph:add_edge
+  Graph::node_id first_node = NULL, node_sommet_courant;
   guchar r;
-  guchar new_r;
 
 ////////////////////////////////////////////////////////////////////////////////
-// Cration du graphe
+// Graph creation.
 
-  // Dfinition de l'espace  visiter selon si on veut une texture "tileable"
+  // Let's define how much space we need to visit, depending on whether we want
+  // a tileable texture.
 
   if (make_tileable) {
     x_inf = patch_posn[0];
@@ -137,8 +126,8 @@ decoupe_graphe (int* patch_posn,
    *                 ______________________
    */
 
-  // On compte le nombre de sommets en parcourant
-  // la rgion commune au patch et  l'image remplie.
+  // We count the number of nodes by visiting the intersection between the
+  // patch and the filled in image.
 
 //   nb_sommets = 0;
 
@@ -149,15 +138,15 @@ decoupe_graphe (int* patch_posn,
 //       r = rempli[x_i][y_i];
 //       if (r) {
 //         nb_sommets++;
-// sera dcomment quand on prendra  nouveau en compte les anciennes coupes
+// We'll uncomment this when we start taking previous cuts into account again.
 //         if (HAS_CUT_NORTH(r)) nb_sommets++;
 //         if (HAS_CUT_WEST(r))  nb_sommets++;
 //       }
 //     }
 //   }
 
-  // On commence par parcourir tout le patch une premire fois pour crer les noeuds
-  // et faire les liens dans node_of_pixel
+  // Start by visiting the whole patch to create nodes and create links in
+  // node_of_pixel.
 
   for (real_x_i = x_inf;
        real_x_i < x_sup;
@@ -178,32 +167,38 @@ decoupe_graphe (int* patch_posn,
     }
   }
 
-  // On cre les arcs.
+  // Create the edges.
   /*
-  On relie  la source les pixels  la fois remplis et au bord du patch
-    (et, dans le cas sans make_tileable, qui de plus ne sont pas au bord de l'image).
-  On relie au puits les pixels remplis dont un voisin n'est pas rempli.
+  We link to the source the pixels that are at the same time filled and also
+  on the edges of the patch (and, for a non tileable texture, that are also
+  not on the edge of the image).
+  We link to the sink the pixels that have at least one neighbor that isn't
+  filled yet.
 
   **********************************************
 
-  Synopsis de la boucle :
+  Loop summary:
 
-  Pour chaque x du patch (intersection avec l'image dans le cas !make_tileable)
-   Pour chaque y du patch (mme remarque)
-    Si je suis rempli
-     Crer les arcs avec mes voisins nord et ouest (s'ils existent dans le patch)
-     (plus tard en grant les anciennes coupes)
-     Si je suis au bord du patch (ie personne dans le patch au nord OU au sud OU  l'est OU  l'ouest)
-     Et dans le cas !make_tileable, si de plus je ne suis pas au bord de l'image (1)
-      Me relier  la source
-     Si l'un de mes voisins (nord, sud, est, ouest) existe (dans le patch ET dans l'image) et n'est pas rempli
-      Me relier au puits
-    Si je ne suis pas rempli
-     Ne rien faire
+  For each x of the patch (intersection with the image if !make_tileable).
+    For each y of the patch (same note)
+    If I am already filled
+     Create the edges with my North and West neighbord (if they exist in the
+         patch) (later we'll need to take previous cuts into account)
+     If I am on the edge of the patch (i.e. there's no other pixel in the patch
+         to the North OR South OR East OR West)
+     And in the !make_tileable case, if I am also not on the edge of the image
+        (1)
+       Then link me to the source
+     If one of my neighbords (North, South, East, West) exists (in the patch
+         AND in the image) and hasn't been filled yet
+       Then link me to the sink
+    If I haven't been filled yet
+      Don't do anything.
 
-  //Le test (1) ci dessus peut faire en sorte qu'il n'y ait aucun pixel reli  la source;
-  //la ligne suivante pallie ce pb.
-  Si !make_tileable, relier le pixel haut gauche de l'intersection (le premier cr)  la source.
+  // The test (1) above might cause the source to not be linked to any pixel.
+  // The following line fixes that problem.
+  If !make_tileable, link the top left pixel of the intersection (the first one
+  that was created) to the source.
   */
 
   for (real_x_i = x_inf;
@@ -218,15 +213,16 @@ decoupe_graphe (int* patch_posn,
       y_p = real_y_i - patch_posn[1];
       y_i = modulo (real_y_i, height_i);
 
-      // Si le pixel de l'image n'est pas rempli, on ne fait rien et on passe au suivant
+      // If the pixel in the image hasn't been filled, we do nothing and skip
+      // to the next one.
       if (!rempli[x_i][y_i]) {
         continue;
       } else {
-        // Cration du noeud et liens
+        // Create the nodes and edges.
         node_sommet_courant = node_of_pixel[x_p * height_p + y_p];
 
-        // Si le voisin nord existe dans le patch et si le pixel nord
-        // est rempli dans l'image, on cre un lien vers lui
+        // If the neighbord exists in the patch and if the pixel to the North
+        // is filled in the image, create a link to it.
         if ((!make_tileable && y_p != 0 && y_i != 0 && rempli[x_i][y_i - 1])
           || (make_tileable && y_p != 0 && rempli[x_i][modulo (y_i - 1, height_i)])) {
           poids = edge_weight (channels,
@@ -239,8 +235,8 @@ decoupe_graphe (int* patch_posn,
                             poids, poids);
         }
 
-        // Si le voisin ouest existe dans le patch et si le pixel ouest
-        // est rempli dans l'image, on cre un lien avec lui
+        // If the West neighbor exists in the patch and if the West pixel is
+        // filled in the image, we create a link to it.
         if ((!make_tileable && x_p != 0 && x_i != 0 && rempli[x_i - 1][y_i])
           || (make_tileable && x_p != 0 && rempli[modulo (x_i - 1, width_i)][y_i])) {
           poids = edge_weight (channels,
@@ -253,45 +249,45 @@ decoupe_graphe (int* patch_posn,
                             poids, poids);
         }
 
-        // Si je suis au bord du patch et si en plus, dans le cas !make_tileable,
-        // je ne suis pas au bord de l'image, me relier  la source
-        if (    (make_tileable && (x_p == 0 || y_p == 0 || x_p == width_p - 1 || y_p == height_p - 1))
+        // If I am on the edge of the patch and, if !make_tileable, I am not on
+        // the edge of the image, link me to the source.
+       if (    (make_tileable && (x_p == 0 || y_p == 0 || x_p == width_p - 1 || y_p == height_p - 1))
             || (!make_tileable && (x_p == 0 || y_p == 0 || x_p == width_p - 1 || y_p == height_p - 1)
 		               &&  x_i != 0 && y_i != 0 && x_i != width_i - 1 && y_i != height_i - 1)) {
           graphe->add_tweights (node_sommet_courant, MAX_CAPACITY, 0);
 	}
 
-        // Si l'un de mes voisins existe et n'est pas rempli, me relier au puits
+        // If one of my neighbords exists and isn't filled, link me to the sink.
         if (((!make_tileable)
-              && (  (y_p != 0            && y_i != 0            && !rempli[x_i][y_i - 1])      // Nord
-                 || (y_p != height_p - 1 && y_i != height_i - 1 && !rempli[x_i][y_i + 1])      // Sud
-                 || (x_p != width_p - 1  && x_i != width_i - 1  && !rempli[x_i + 1][y_i])      // Est
-                 || (x_p != 0            && x_i != 0            && !rempli[x_i - 1][y_i])))    // Ouest
+              && (  (y_p != 0            && y_i != 0            && !rempli[x_i][y_i - 1])      // North
+                 || (y_p != height_p - 1 && y_i != height_i - 1 && !rempli[x_i][y_i + 1])      // South
+                 || (x_p != width_p - 1  && x_i != width_i - 1  && !rempli[x_i + 1][y_i])      // East
+                 || (x_p != 0            && x_i != 0            && !rempli[x_i - 1][y_i])))    // West
             || ((make_tileable)
-              && (  (y_p != 0            && !rempli[x_i][modulo (y_i - 1, height_i)])          // Nord
-                 || (y_p != height_p - 1 && !rempli[x_i][modulo (y_i + 1, height_i)])          // Sud
-                 || (x_p != width_p - 1  && !rempli[modulo (x_i + 1, width_i)][y_i])           // Est
-                 || (x_p != 0            && !rempli[modulo (x_i - 1, width_i)][y_i])))) {      // Ouest
-	  //	  printf ("Connecting %i, %i to Sink\n", x_p, y_p);
+              && (  (y_p != 0            && !rempli[x_i][modulo (y_i - 1, height_i)])          // North
+                 || (y_p != height_p - 1 && !rempli[x_i][modulo (y_i + 1, height_i)])          // South
+                 || (x_p != width_p - 1  && !rempli[modulo (x_i + 1, width_i)][y_i])           // East
+                 || (x_p != 0            && !rempli[modulo (x_i - 1, width_i)][y_i])))) {      // West
           graphe->add_tweights (node_sommet_courant, 0, MAX_CAPACITY);
 	}
       }
     }
   }
 
-  // Si !make_tileable, on relie  la source le pixel haut gauche de patch \cap image
+  // If !make_tileable, link the top left pixel in patch \cap image to the
+  // source.
   if (!make_tileable) {
     graphe->add_tweights (first_node, MAX_CAPACITY, 0);
   }
 
 
 ////////////////////////////////////////////////////////////////////////////////
-// Calcul de la coupe
+// Compute the cut.
 
-  graphe->maxflow ();
+  graphe->maxflow();
 
 ////////////////////////////////////////////////////////////////////////////////
-// Mise_a_jour de l'image
+// Update the image.
 
   for (real_x_i = x_inf; real_x_i < x_sup; real_x_i++) {
     x_p = real_x_i - patch_posn[0];
@@ -316,9 +312,10 @@ decoupe_graphe (int* patch_posn,
   }
 
 ////////////////////////////////////////////////////////////////////////////////
-//On nettoie tout
+// Clean up.
 
   delete graphe;
+  free (node_of_pixel);
 
   return;
 }
