@@ -1,10 +1,12 @@
 #include <stdlib.h>
 
-#include <libgimp/gimp.h>
+#include <glib.h>
+
 #include "texturize.h"
 
-guchar **
-init_guchar_tab_2d (gint x, gint y) {
+// Allocates enough memory for a 2-dimensional table of guchars and
+// initializes all elements to zero.
+guchar ** init_guchar_tab_2d (gint x, gint y) {
   guchar ** tab;
   gint i, j;
   tab = (guchar**) malloc (x * sizeof (guchar*));
